@@ -1,10 +1,5 @@
-/*
- * Copyright (C) 2007, 2008 Stig Venaas <venaas@uninett.no>
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- */
+/* Copyright (c) 2007-2008, UNINETT AS */
+/* See LICENSE for licensing information. */
 
 #include <string.h>
 #include <stdarg.h>
@@ -541,6 +536,7 @@ int getgenericconfig(struct gconffile **cf, char *block, ...) {
 errparam:
     debug(DBG_ERR, "getgenericconfig: internal parameter error");
 errexit:
+    va_end(ap);
     free(opt);
     free(val);
     return 0;
