@@ -1,10 +1,5 @@
-/*
- * Copyright (C) 2008 Stig Venaas <venaas@uninett.no>
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- */
+/* Copyright (c) 2008, UNINETT AS */
+/* See LICENSE for licensing information. */
 
 #include <stdlib.h>
 #include <string.h>
@@ -92,6 +87,7 @@ void *hash_read(struct hash *h, void *key, uint32_t keylen) {
 void *hash_extract(struct hash *h, void *key, uint32_t keylen) {
     struct list_node *ln;
     struct hash_entry *e;
+    void *data;
 
     if (!h)
 	return 0;
@@ -101,9 +97,10 @@ void *hash_extract(struct hash *h, void *key, uint32_t keylen) {
 	if (e->keylen == keylen && !memcmp(e->key, key, keylen)) {
 	    free(e->key);
 	    list_removedata(h->hashlist, e);
+	    data = e->data;
 	    free(e);
 	    pthread_mutex_unlock(&h->mutex);
-	    return e->data;
+	    return data;
 	}
     }
     pthread_mutex_unlock(&h->mutex);
