@@ -1,15 +1,6 @@
-/*
- * Copyright (C) 2006-2009 Stig Venaas <venaas@uninett.no>
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- */
-
-/* Code contributions from:
- *
- * Simon Leinen <simon.leinen@switch.ch>
- */
+/* Copyright (c) 2007-2009, UNINETT AS
+ * Copyright (c) 2012,2016 NORDUnet A/S */
+/* See LICENSE for licensing information. */
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -134,6 +125,7 @@ errexit:
 
 int resolvehostport(struct hostportres *hp, int af, int socktype, uint8_t passive) {
     struct addrinfo hints, *res;
+    char tmp[INET6_ADDRSTRLEN];
 
     memset(&hints, 0, sizeof(hints));
     hints.ai_socktype = socktype;
@@ -172,7 +164,10 @@ int resolvehostport(struct hostportres *hp, int af, int socktype, uint8_t passiv
 	    }
 	}
     }
-    debug(DBG_DBG, "%s: %s -> %s", __func__, hp->host, addr2string(hp->addrinfo->ai_addr));
+    debug(DBG_DBG, "%s: %s -> %s", __func__,
+          (hp->host ? hp->host : "(src info not available)"),
+          ((hp->addrinfo && hp->addrinfo->ai_addr) ?
+           addr2string(hp->addrinfo->ai_addr, tmp, sizeof(tmp)) : "(dst info not available)"));
     return 1;
 
 errexit:
