@@ -1,10 +1,6 @@
-/*
- * Copyright (C) 2006-2008 Stig Venaas <venaas@uninett.no>
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- */
+/* Copyright (c) 2007-2008, UNINETT AS */
+/* Copyright (c) 2015, NORDUnet A/S */
+/* See LICENSE for licensing information. */
 
 #define RAD_Access_Request 1
 #define RAD_Access_Accept 2
@@ -17,10 +13,12 @@
 
 #define RAD_Attr_User_Name 1
 #define RAD_Attr_User_Password 2
+#define RAD_Attr_CHAP_Password 3
 #define RAD_Attr_Reply_Message 18
 #define RAD_Attr_Vendor_Specific 26
 #define RAD_Attr_Calling_Station_Id 31
-#define RAD_Proxy_State 33
+#define RAD_Attr_Proxy_State 33
+#define RAD_Attr_CHAP_Challenge 60
 #define RAD_Attr_Tunnel_Password 69
 #define RAD_Attr_Message_Authenticator 80
 
