@@ -1,11 +1,6 @@
-/*
- * Copyright (C) 2006-2009 Stig Venaas <venaas@uninett.no>
- * Copyright (C) 2010,2011,2012 NORDUnet A/S
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- */
+/* Copyright (c) 2007-2009, UNINETT AS
+ * Copyright (c) 2010-2012,2016, NORDUnet A/S */
+/* See LICENSE for licensing information. */
 
 #include <sys/time.h>
 #include <stdint.h>
@@ -29,8 +24,25 @@
 #define STATUS_SERVER_PERIOD 25
 #define IDLE_TIMEOUT 300
 
+/* We want PTHREAD_STACK_SIZE to be 32768, but some platforms
+ * have a higher minimum value defined in PTHREAD_STACK_MIN. */
+#define PTHREAD_STACK_SIZE 32768
+#if defined(PTHREAD_STACK_MIN)
+#if PTHREAD_STACK_MIN > PTHREAD_STACK_SIZE
+#undef PTHREAD_STACK_SIZE
+#define PTHREAD_STACK_SIZE PTHREAD_STACK_MIN
+#endif
+#endif
+
+/* For systems that only support RFC 2292 Socket API, but not RFC 3542
+ * like Cygwin */
+#ifndef IPV6_RECVPKTINFO
+#define IPV6_RECVPKTINFO IPV6_PKTINFO
+#endif
+
 /* 27262 is vendor DANTE Ltd. */
 #define DEFAULT_TTL_ATTR "27262:1"
+#define DEFAULT_FTICKS_PREFIX "F-TICKS/eduroam/1.0"
 
 #define RAD_UDP 0
 #define RAD_TLS 1
@@ -44,26 +56,38 @@ enum rsp_fticks_reporting_type {
     RSP_FTICKS_REPORTING_FULL
 };
 
-enum rsp_fticks_mac_type {
-    RSP_FTICKS_MAC_STATIC = 0,
-    RSP_FTICKS_MAC_ORIGINAL,
-    RSP_FTICKS_MAC_VENDOR_HASHED,
-    RSP_FTICKS_MAC_VENDOR_KEY_HASHED, /* Default.  */
-    RSP_FTICKS_MAC_FULLY_HASHED,
-    RSP_FTICKS_MAC_FULLY_KEY_HASHED
+enum rsp_mac_type {
+    RSP_MAC_STATIC = 0,
+    RSP_MAC_ORIGINAL,
+    RSP_MAC_VENDOR_HASHED,
+    RSP_MAC_VENDOR_KEY_HASHED, /* Default.  */
+    RSP_MAC_FULLY_HASHED,
+    RSP_MAC_FULLY_KEY_HASHED
+};
+
+enum rsp_server_state {
+    RSP_SERVER_STATE_STARTUP = 0, /* default */
+    RSP_SERVER_STATE_CONNECTED,
+    RSP_SERVER_STATE_RECONNECTING,
+    RSP_SERVER_STATE_FAILING
 };
 
 struct options {
     char *pidfile;
     char *logdestination;
     char *ftickssyslogfacility;
+    char *fticksprefix;
     char *ttlattr;
     uint32_t ttlattrtype[2];
     uint8_t addttl;
     uint8_t loglevel;
+	uint8_t logtid;
+	uint8_t logfullusername;
     uint8_t loopprevention;
+	enum rsp_mac_type log_mac;
+	uint8_t *log_key;
     enum rsp_fticks_reporting_type fticks_reporting;
-    enum rsp_fticks_mac_type fticks_mac;
+    enum rsp_mac_type fticks_mac;
     uint8_t *fticks_key;
     uint8_t ipv4only;
     uint8_t ipv6only;
@@ -77,6 +101,7 @@ struct commonprotoopts {
 struct request {
     struct timeval created;
     uint32_t refcount;
+	pthread_mutex_t refmutex;
     uint8_t *buf, *replybuf;
     struct radmsg *msg;
     struct client *from;
@@ -86,7 +111,6 @@ struct request {
     uint8_t rqauth[16];
     uint8_t newid;
     int udpsock; /* only for UDP */
-    uint16_t udpport; /* only for UDP */
 };
 
 /* requests that our client will send */
@@ -127,6 +151,7 @@ struct clsrvconf {
     uint8_t dupinterval;
     uint8_t certnamecheck;
     uint8_t addttl;
+    uint8_t keepalive;
     uint8_t loopprevention;
     struct rewrite *rewritein;
     struct rewrite *rewriteout;
@@ -144,9 +169,9 @@ struct client {
     struct clsrvconf *conf;
     int sock;
     SSL *ssl;
+	pthread_mutex_t lock;
     struct request *rqs[MAX_REQUESTS];
     struct gqueue *replyq;
-    struct gqueue *rbios; /* for dtls */
     struct sockaddr *addr;
     time_t expiry; /* for udp */
 };
@@ -160,21 +185,16 @@ struct server {
     uint8_t clientrdgone;
     struct timeval lastconnecttry;
     struct timeval lastreply;
-    uint8_t connectionok;
+    enum rsp_server_state state;
     uint8_t lostrqs;
-    uint8_t dynstartup;
-    uint8_t dynfailing;
-#if defined ENABLE_EXPERIMENTAL_DYNDISC
-    uint8_t in_use;
-#endif
     char *dynamiclookuparg;
     int nextid;
     struct timeval lastrcv;
     struct rqout *requests;
     uint8_t newrq;
+	uint8_t conreset;
     pthread_mutex_t newrq_mutex;
     pthread_cond_t newrq_cond;
-    struct gqueue *rbios; /* for dtls */
 };
 
 struct realm {
@@ -183,6 +203,7 @@ struct realm {
     uint8_t accresp;
     regex_t regex;
     uint32_t refcount;
+    pthread_mutex_t refmutex;
     pthread_mutex_t mutex;
     struct realm *parent;
     struct list *subrealms;
@@ -239,13 +260,13 @@ struct client *addclient(struct clsrvconf *conf, uint8_t lock);
 void removelockedclient(struct client *client);
 void removeclient(struct client *client);
 struct gqueue *newqueue();
-void freebios(struct gqueue *q);
 struct request *newrequest();
 void freerq(struct request *rq);
 int radsrv(struct request *rq);
 void replyh(struct server *server, unsigned char *buf);
 struct addrinfo *resolve_hostport_addrinfo(uint8_t type, char *hostport);
 uint8_t *radattr2ascii(struct tlv *attr);
+pthread_attr_t pthread_attr;
 
 /* Local Variables: */
 /* c-file-style: "stroustrup" */
