@@ -1,10 +1,6 @@
-/*
- * Copyright (C) 2008-2009 Stig Venaas <venaas@uninett.no>
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- */
+/* Copyright (c) 2007-2009, UNINETT AS
+ * Copyright (c) 2012, NORDUnet A/S */
+/* See LICENSE for licensing information. */
 
 #include <signal.h>
 #include <sys/socket.h>
@@ -18,7 +14,7 @@
 #endif
 #include <sys/time.h>
 #include <sys/types.h>
-#include <sys/select.h>
+#include <poll.h>
 #include <ctype.h>
 #include <sys/wait.h>
 #include <arpa/inet.h>
@@ -84,50 +80,66 @@ void tcpsetsrcres() {
 }
 
 int tcpconnect(struct server *server, struct timeval *when, int timeout, char *text) {
-    struct timeval now;
-    time_t elapsed;
+    struct timeval now, start = {0,0};
+    time_t wait;
 
     debug(DBG_DBG, "tcpconnect: called from %s", text);
     pthread_mutex_lock(&server->lock);
-    if (when && memcmp(&server->lastconnecttry, when, sizeof(struct timeval))) {
-	/* already reconnected, nothing to do */
-	debug(DBG_DBG, "tcpconnect(%s): seems already reconnected", text);
-	pthread_mutex_unlock(&server->lock);
-	return 1;
+
+    if (server->state == RSP_SERVER_STATE_CONNECTED)
+        server->state = RSP_SERVER_STATE_RECONNECTING;
+
+    gettimeofday(&now, NULL);
+    if (when && (now.tv_sec - when->tv_sec) < 30 ) {
+        /* last connection was less than 30s ago. Delay next attempt */
+        start.tv_sec = now.tv_sec + 30 - (now.tv_sec - when->tv_sec);
     }
 
     for (;;) {
-	gettimeofday(&now, NULL);
-	elapsed = now.tv_sec - server->lastconnecttry.tv_sec;
-	if (timeout && server->lastconnecttry.tv_sec && elapsed > timeout) {
-	    debug(DBG_DBG, "tcpconnect: timeout");
-	    if (server->sock >= 0)
-		close(server->sock);
-	    pthread_mutex_unlock(&server->lock);
-	    return 0;
-	}
-	if (server->connectionok) {
-	    server->connectionok = 0;
-	    sleep(2);
-	} else if (elapsed < 1)
-	    sleep(2);
-	else if (elapsed < 60) {
-	    debug(DBG_INFO, "tcpconnect: sleeping %lds", elapsed);
-	    sleep(elapsed);
-	} else if (elapsed < 100000) {
-	    debug(DBG_INFO, "tcpconnect: sleeping %ds", 60);
-	    sleep(60);
-	} else
-	    server->lastconnecttry.tv_sec = now.tv_sec;  /* no sleep at startup */
+        if (server->sock >= 0)
+            close(server->sock);
+        server->sock = -1;
 
-	if (server->sock >= 0)
-	    close(server->sock);
-	if ((server->sock = connecttcphostlist(server->conf->hostports, srcres)) >= 0)
-	    break;
+        /* no sleep at startup or at first try */
+        if (start.tv_sec) {
+            gettimeofday(&now, NULL);
+            wait = abs(now.tv_sec - start.tv_sec);
+            wait = wait > 60 ? 60 : wait;
+
+            if (timeout && (now.tv_sec - start.tv_sec) > timeout) {
+                debug(DBG_DBG, "tlsconnect: timeout");
+                pthread_mutex_unlock(&server->lock);
+                return 0;
+            }
+
+            /* give up lock while sleeping for next try */
+            pthread_mutex_unlock(&server->lock);
+            if (wait < 1)
+                sleep(2);
+            else {
+                debug(DBG_INFO, "Next connection attempt to %s in %lds", server->conf->name, wait);
+                sleep(wait);
+            }
+            pthread_mutex_lock(&server->lock);
+            debug(DBG_INFO, "tlsconnect: retry connecting to %s", server->conf->name);
+        } else {
+            gettimeofday(&start, NULL);
+        }
+
+        if ((server->sock = connecttcphostlist(server->conf->hostports, srcres)) < 0)
+            continue;
+        if (server->conf->keepalive)
+            enable_keepalive(server->sock);
+        break;
     }
-    server->connectionok = 1;
+    server->state = RSP_SERVER_STATE_CONNECTED;
     gettimeofday(&server->lastconnecttry, NULL);
+    server->lostrqs = 0;
     pthread_mutex_unlock(&server->lock);
+    pthread_mutex_lock(&server->newrq_mutex);
+    server->conreset = 1;
+    pthread_cond_signal(&server->newrq_cond);
+    pthread_mutex_unlock(&server->newrq_mutex);
     return 1;
 }
 
@@ -135,24 +147,21 @@ int tcpconnect(struct server *server, struct timeval *when, int timeout, char *t
 /* returns 0 on timeout, -1 on error and num if ok */
 int tcpreadtimeout(int s, unsigned char *buf, int num, int timeout) {
     int ndesc, cnt, len;
-    fd_set readfds, writefds;
-    struct timeval timer;
+    struct pollfd fds[1];
 
     if (s < 0)
 	return -1;
     /* make socket non-blocking? */
     for (len = 0; len < num; len += cnt) {
-	FD_ZERO(&readfds);
-	FD_SET(s, &readfds);
-	writefds = readfds;
-	if (timeout) {
-	    timer.tv_sec = timeout;
-	    timer.tv_usec = 0;
-	}
-	ndesc = select(s + 1, &readfds, &writefds, NULL, timeout ? &timer : NULL);
+        fds[0].fd = s;
+        fds[0].events = POLLIN;
+	ndesc = poll(fds, 1, timeout? timeout * 1000 : -1);
 	if (ndesc < 1)
 	    return ndesc;
 
+    if (fds[0].revents & (POLLERR | POLLHUP | POLLNVAL) ) {
+        return -1;
+    }
 	cnt = read(s, buf + len, num - len);
 	if (cnt <= 0)
 	    return -1;
@@ -173,6 +182,10 @@ unsigned char *radtcpget(int s, int timeout) {
 	}
 
 	len = RADLEN(buf);
+	if (len < 4) {
+	    debug(DBG_ERR, "radtcpget: length too small");
+	    continue;
+	}
 	rad = malloc(len);
 	if (!rad) {
 	    debug(DBG_ERR, "radtcpget: malloc failed");
@@ -203,7 +216,7 @@ int clientradputtcp(struct server *server, unsigned char *rad) {
     size_t len;
     struct clsrvconf *conf = server->conf;
 
-    if (!server->connectionok)
+    if (server->state != RSP_SERVER_STATE_CONNECTED)
 	return 0;
     len = RADLEN(rad);
     if ((cnt = write(server->sock, rad, len)) <= 0) {
@@ -222,8 +235,10 @@ void *tcpclientrd(void *arg) {
     for (;;) {
 	/* yes, lastconnecttry is really necessary */
 	lastconnecttry = server->lastconnecttry;
-	buf = radtcpget(server->sock, 0);
+	buf = radtcpget(server->sock, server->dynamiclookuparg ? IDLE_TIMEOUT : 0);
 	if (!buf) {
+        if (server->dynamiclookuparg)
+		break;
 	    tcpconnect(server, &lastconnecttry, 0, "tcpclientrd");
 	    continue;
 	}
@@ -231,6 +246,9 @@ void *tcpclientrd(void *arg) {
 	replyh(server, buf);
     }
     server->clientrdgone = 1;
+    pthread_mutex_lock(&server->newrq_mutex);
+    pthread_cond_signal(&server->newrq_cond);
+    pthread_mutex_unlock(&server->newrq_mutex);
     return NULL;
 }
 
@@ -239,8 +257,9 @@ void *tcpserverwr(void *arg) {
     struct client *client = (struct client *)arg;
     struct gqueue *replyq;
     struct request *reply;
+    char tmp[INET6_ADDRSTRLEN];
 
-    debug(DBG_DBG, "tcpserverwr: starting for %s", addr2string(client->addr));
+    debug(DBG_DBG, "tcpserverwr: starting for %s", addr2string(client->addr, tmp, sizeof(tmp)));
     replyq = client->replyq;
     for (;;) {
 	pthread_mutex_lock(&replyq->mutex);
@@ -262,9 +281,9 @@ void *tcpserverwr(void *arg) {
 	cnt = write(client->sock, reply->replybuf, RADLEN(reply->replybuf));
 	if (cnt > 0)
 	    debug(DBG_DBG, "tcpserverwr: sent %d bytes, Radius packet of length %d to %s",
-		  cnt, RADLEN(reply->replybuf), addr2string(client->addr));
+		  cnt, RADLEN(reply->replybuf), addr2string(client->addr, tmp, sizeof(tmp)));
 	else
-	    debug(DBG_ERR, "tcpserverwr: write error for %s", addr2string(client->addr));
+	    debug(DBG_ERR, "tcpserverwr: write error for %s", addr2string(client->addr, tmp, sizeof(tmp)));
 	freerq(reply);
     }
 }
@@ -273,8 +292,9 @@ void tcpserverrd(struct client *client) {
     struct request *rq;
     uint8_t *buf;
     pthread_t tcpserverwrth;
+    char tmp[INET6_ADDRSTRLEN];
 
-    debug(DBG_DBG, "tcpserverrd: starting for %s", addr2string(client->addr));
+    debug(DBG_DBG, "tcpserverrd: starting for %s", addr2string(client->addr, tmp, sizeof(tmp)));
 
     if (pthread_create(&tcpserverwrth, &pthread_attr, tcpserverwr, (void *)client)) {
 	debug(DBG_ERR, "tcpserverrd: pthread_create failed");
@@ -284,10 +304,10 @@ void tcpserverrd(struct client *client) {
     for (;;) {
 	buf = radtcpget(client->sock, 0);
 	if (!buf) {
-	    debug(DBG_ERR, "tcpserverrd: connection from %s lost", addr2string(client->addr));
+	    debug(DBG_ERR, "tcpserverrd: connection from %s lost", addr2string(client->addr, tmp, sizeof(tmp)));
 	    break;
 	}
-	debug(DBG_DBG, "tcpserverrd: got Radius message from %s", addr2string(client->addr));
+	debug(DBG_DBG, "tcpserverrd: got Radius message from %s", addr2string(client->addr, tmp, sizeof(tmp)));
 	rq = newrequest();
 	if (!rq) {
 	    free(buf);
@@ -296,7 +316,7 @@ void tcpserverrd(struct client *client) {
 	rq->buf = buf;
 	rq->from = client;
 	if (!radsrv(rq)) {
-	    debug(DBG_ERR, "tcpserverrd: message authentication/validation failed, closing connection from %s", addr2string(client->addr));
+	    debug(DBG_ERR, "tcpserverrd: message authentication/validation failed, closing connection from %s", addr2string(client->addr, tmp, sizeof(tmp)));
 	    break;
 	}
     }
@@ -308,7 +328,7 @@ void tcpserverrd(struct client *client) {
     pthread_mutex_unlock(&client->replyq->mutex);
     debug(DBG_DBG, "tcpserverrd: waiting for writer to end");
     pthread_join(tcpserverwrth, NULL);
-    debug(DBG_DBG, "tcpserverrd: reader for %s exiting", addr2string(client->addr));
+    debug(DBG_DBG, "tcpserverrd: reader for %s exiting", addr2string(client->addr, tmp, sizeof(tmp)));
 }
 void *tcpservernew(void *arg) {
     int s;
@@ -316,26 +336,30 @@ void *tcpservernew(void *arg) {
     socklen_t fromlen = sizeof(from);
     struct clsrvconf *conf;
     struct client *client;
+    char tmp[INET6_ADDRSTRLEN];
 
     s = *(int *)arg;
+    free(arg);
     if (getpeername(s, (struct sockaddr *)&from, &fromlen)) {
 	debug(DBG_DBG, "tcpservernew: getpeername failed, exiting");
 	goto exit;
     }
-    debug(DBG_WARN, "tcpservernew: incoming TCP connection from %s", addr2string((struct sockaddr *)&from));
+    debug(DBG_WARN, "tcpservernew: incoming TCP connection from %s", addr2string((struct sockaddr *)&from, tmp, sizeof(tmp)));
 
     conf = find_clconf(handle, (struct sockaddr *)&from, NULL);
     if (conf) {
-	client = addclient(conf, 1);
-	if (client) {
-	    client->sock = s;
-	    client->addr = addr_copy((struct sockaddr *)&from);
-	    tcpserverrd(client);
-	    removeclient(client);
-	} else
-	    debug(DBG_WARN, "tcpservernew: failed to create new client instance");
+        client = addclient(conf, 1);
+        if (client) {
+            if(conf->keepalive)
+                enable_keepalive(s);
+            client->sock = s;
+            client->addr = addr_copy((struct sockaddr *)&from);
+            tcpserverrd(client);
+            removeclient(client);
+        } else
+            debug(DBG_WARN, "tcpservernew: failed to create new client instance");
     } else
-	debug(DBG_WARN, "tcpservernew: ignoring request, no matching TCP client");
+        debug(DBG_WARN, "tcpservernew: ignoring request, no matching TCP client");
 
 exit:
     shutdown(s, SHUT_RDWR);
@@ -345,11 +369,11 @@ exit:
 
 void *tcplistener(void *arg) {
     pthread_t tcpserverth;
-    int s, *sp = (int *)arg;
+    int s, *sp = (int *)arg, *s_arg = NULL;
     struct sockaddr_storage from;
     socklen_t fromlen = sizeof(from);
 
-    listen(*sp, 0);
+    listen(*sp, 128);
 
     for (;;) {
 	s = accept(*sp, (struct sockaddr *)&from, &fromlen);
@@ -357,8 +381,13 @@ void *tcplistener(void *arg) {
 	    debug(DBG_WARN, "accept failed");
 	    continue;
 	}
-	if (pthread_create(&tcpserverth, &pthread_attr, tcpservernew, (void *)&s)) {
+        s_arg = malloc(sizeof(s));
+        if (!s_arg)
+            debugx(1, DBG_ERR, "malloc failed");
+        *s_arg = s;
+	if (pthread_create(&tcpserverth, &pthread_attr, tcpservernew, (void *) s_arg)) {
 	    debug(DBG_ERR, "tcplistener: pthread_create failed");
+            free(s_arg);
 	    shutdown(s, SHUT_RDWR);
 	    close(s);
 	    continue;
