@@ -1,13 +1,10 @@
-/*
- * Copyright (C) 2006-2009 Stig Venaas <venaas@uninett.no>
- * Copyright (C) 2010,2011 NORDUnet A/S
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- */
+/* Copyright (c) 2007-2009, UNINETT AS
+ * Copyright (c) 2010-2011,2015-2016, NORDUnet A/S */
+/* See LICENSE for licensing information. */
 
 #if defined(RADPROT_TLS) || defined(RADPROT_DTLS)
+#define _GNU_SOURCE
+#include <stdio.h>
 #include <signal.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -15,12 +12,10 @@
 #include <string.h>
 #include <unistd.h>
 #include <limits.h>
-#ifdef SYS_SOLARIS9
 #include <fcntl.h>
-#endif
+#include <poll.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#include <sys/select.h>
 #include <ctype.h>
 #include <sys/wait.h>
 #include <arpa/inet.h>
@@ -40,6 +35,48 @@
 
 static struct hash *tlsconfs = NULL;
 
+#define COOKIE_SECRET_LENGTH 16
+static unsigned char cookie_secret[COOKIE_SECRET_LENGTH];
+static uint8_t cookie_secret_initialized = 0;
+
+
+/* callbacks for making OpenSSL < 1.1 thread safe */
+#if OPENSSL_VERSION_NUMBER < 0x10100000
+static pthread_mutex_t *ssl_locks = NULL;
+
+unsigned long ssl_thread_id() {
+    return (unsigned long)pthread_self();
+}
+
+void ssl_locking_callback(int mode, int type, const char *file, int line) {
+    if (mode & CRYPTO_LOCK)
+       pthread_mutex_lock(&ssl_locks[type]);
+    else
+       pthread_mutex_unlock(&ssl_locks[type]);
+}
+#endif
+
+void sslinit() {
+#if OPENSSL_VERSION_NUMBER < 0x10100000
+    int i;
+
+    SSL_library_init();
+
+    ssl_locks = OPENSSL_malloc(CRYPTO_num_locks() * sizeof(pthread_mutex_t));
+    if (!ssl_locks)
+        debugx(1, DBG_ERR, "malloc failed");
+
+    for (i = 0; i < CRYPTO_num_locks(); i++) {
+        pthread_mutex_init(&ssl_locks[i], NULL);
+    }
+    CRYPTO_set_id_callback(ssl_thread_id);
+    CRYPTO_set_locking_callback(ssl_locking_callback);
+    SSL_load_error_strings();
+#else
+    OPENSSL_init_ssl(0, NULL);
+#endif
+}
+
 static int pem_passwd_cb(char *buf, int size, int rwflag, void *userdata) {
     int pwdlen = strlen(userdata);
     if (rwflag != 0 || pwdlen > size) /* not for decryption or too large */
@@ -102,6 +139,94 @@ static int verify_cb(int ok, X509_STORE_CTX *ctx) {
     return ok;
 }
 
+static int cookie_calculate_hash(struct sockaddr *peer, time_t time, uint8_t *result, unsigned int *resultlength) {
+    uint8_t *buf;
+    int length;
+
+    length = SOCKADDRP_SIZE(peer) + sizeof(time_t);
+    buf = OPENSSL_malloc(length);
+    if (!buf) {
+        debug(DBG_ERR, "cookie_calculate_hash: malloc failed");
+        return 0;
+    }
+
+    memcpy(buf, &time, sizeof(time_t));
+    memcpy(buf+sizeof(time_t), peer, SOCKADDRP_SIZE(peer));
+
+    HMAC(EVP_sha256(), (const void*) cookie_secret, COOKIE_SECRET_LENGTH,
+         buf, length, result, resultlength);
+    OPENSSL_free(buf);
+    return 1;
+}
+
+static int cookie_generate_cb(SSL *ssl, unsigned char *cookie, unsigned int *cookie_len) {
+    struct sockaddr_storage peer;
+    struct timeval now;
+    uint8_t result[EVP_MAX_MD_SIZE];
+    unsigned int resultlength;
+
+    if (!cookie_secret_initialized) {
+        if (!RAND_bytes(cookie_secret, COOKIE_SECRET_LENGTH))
+            debugx(1,DBG_ERR, "cookie_generate_cg: error generating random secret");
+        cookie_secret_initialized = 1;
+    }
+
+    if (BIO_dgram_get_peer(SSL_get_rbio(ssl), &peer) <= 0)
+        return 0;
+    gettimeofday(&now, NULL);
+    if (!cookie_calculate_hash((struct sockaddr *)&peer, now.tv_sec, result, &resultlength))
+        return 0;
+
+    memcpy(cookie, &now.tv_sec, sizeof(time_t));
+    memcpy(cookie + sizeof(time_t), result, resultlength);
+    *cookie_len = resultlength + sizeof(time_t);
+
+    return 1;
+}
+
+#if OPENSSL_VERSION_NUMBER < 0x10100000
+static int cookie_verify_cb(SSL *ssl, unsigned char *cookie, unsigned int cookie_len) {
+#else
+static int cookie_verify_cb(SSL *ssl, const unsigned char *cookie, unsigned int cookie_len) {
+#endif
+    struct sockaddr_storage peer;
+    struct timeval now;
+    time_t cookie_time;
+    uint8_t result[EVP_MAX_MD_SIZE];
+    unsigned int resultlength;
+
+    if (!cookie_secret_initialized)
+        return 0;
+
+    if (cookie_len < sizeof(time_t)) {
+        debug(DBG_DBG, "cookie_verify_cb: cookie too short. ignoring.");
+        return 0;
+    }
+
+    gettimeofday(&now, NULL);
+    cookie_time = *(time_t *)cookie;
+    if (now.tv_sec - cookie_time > 5) {
+        debug(DBG_DBG, "cookie_verify_cb: cookie invalid or older than 5s. ignoring.");
+        return 0;
+    }
+
+    if (BIO_dgram_get_peer(SSL_get_rbio(ssl), &peer) <= 0)
+        return 0;
+    if (!cookie_calculate_hash((struct sockaddr *)&peer, cookie_time, result, &resultlength))
+        return 0;
+
+    if (resultlength + sizeof(time_t) != cookie_len) {
+        debug(DBG_DBG, "cookie_verify_cb: invalid cookie length. ignoring.");
+        return 0;
+    }
+
+    if (memcmp(cookie + sizeof(time_t), result, resultlength)) {
+        debug(DBG_DBG, "cookie_verify_cb: cookie not valid. ignoring.");
+        return 0;
+    }
+    return 1;
+}
+
 #ifdef DEBUG
 static void ssl_info_callback(const SSL *ssl, int where, int ret) {
     const char *s;
@@ -158,6 +283,7 @@ static int tlsaddcacrl(SSL_CTX *ctx, struct tls *conf) {
     X509_STORE *x509_s;
     unsigned long error;
 
+    SSL_CTX_set_cert_store(ctx, X509_STORE_new());
     if (!SSL_CTX_load_verify_locations(ctx, conf->cacertfile, conf->cacertpath)) {
 	while ((error = ERR_get_error()))
 	    debug(DBG_ERR, "SSL: %s", ERR_error_string(error, NULL));
@@ -187,6 +313,8 @@ static int tlsaddcacrl(SSL_CTX *ctx, struct tls *conf) {
 
     SSL_CTX_set_verify(ctx, SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT, verify_cb);
     SSL_CTX_set_verify_depth(ctx, MAX_CERT_DEPTH + 1);
+    SSL_CTX_set_cookie_generate_cb(ctx, cookie_generate_cb);
+    SSL_CTX_set_cookie_verify_cb(ctx, cookie_verify_cb);
 
     if (conf->crlcheck || conf->vpm) {
 	x509_s = SSL_CTX_get_cert_store(ctx);
@@ -203,12 +331,18 @@ static int tlsaddcacrl(SSL_CTX *ctx, struct tls *conf) {
 static SSL_CTX *tlscreatectx(uint8_t type, struct tls *conf) {
     SSL_CTX *ctx = NULL;
     unsigned long error;
-    long sslversion = SSLeay();
 
     switch (type) {
 #ifdef RADPROT_TLS
     case RAD_TLS:
-	ctx = SSL_CTX_new(TLSv1_method());
+#if OPENSSL_VERSION_NUMBER >= 0x10100000
+        /* TLS_method() was introduced in OpenSSL 1.1.0. */
+	ctx = SSL_CTX_new(TLS_method());
+#else
+        /* No TLS_method(), use SSLv23_method() and disable SSLv2 and SSLv3. */
+        ctx = SSL_CTX_new(SSLv23_method());
+        SSL_CTX_set_options(ctx, SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3);
+#endif
 #ifdef DEBUG
 	SSL_CTX_set_info_callback(ctx, ssl_info_callback);
 #endif
@@ -216,7 +350,12 @@ static SSL_CTX *tlscreatectx(uint8_t type, struct tls *conf) {
 #endif
 #ifdef RADPROT_DTLS
     case RAD_DTLS:
+#if OPENSSL_VERSION_NUMBER >= 0x10002000
+        /* DTLS_method() seems to have been introduced in OpenSSL 1.0.2. */
+	ctx = SSL_CTX_new(DTLS_method());
+#else
 	ctx = SSL_CTX_new(DTLSv1_method());
+#endif
 #ifdef DEBUG
 	SSL_CTX_set_info_callback(ctx, ssl_info_callback);
 #endif
@@ -229,6 +368,9 @@ static SSL_CTX *tlscreatectx(uint8_t type, struct tls *conf) {
 	return NULL;
     }
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+    {
+    long sslversion = SSLeay();
     if (sslversion < 0x00908100L ||
         (sslversion >= 0x10000000L && sslversion < 0x10000020L)) {
         debug(DBG_WARN, "%s: %s seems to be of a version with a "
@@ -237,6 +379,8 @@ static SSL_CTX *tlscreatectx(uint8_t type, struct tls *conf) {
 	      __func__, SSLeay_version(SSLEAY_VERSION), ctx);
         SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_OFF);
     }
+    }
+#endif
 
     if (conf->certkeypwd) {
 	SSL_CTX_set_default_passwd_cb_userdata(ctx, conf->certkeypwd);
@@ -327,6 +471,34 @@ SSL_CTX *tlsgetctx(uint8_t type, struct tls *t) {
     return NULL;
 }
 
+void tlsreloadcrls() {
+    struct tls *conf;
+    struct hash_entry *entry;
+    struct timeval now;
+
+    debug (DBG_NOTICE, "reloading CRLs");
+
+    gettimeofday(&now, NULL);
+
+    for (entry = hash_first(tlsconfs); entry; entry = hash_next(entry)) {
+	conf = (struct tls *)entry->data;
+#ifdef RADPROT_TLS
+	if (conf->tlsctx) {
+	    if (conf->tlsexpiry)
+		conf->tlsexpiry = now.tv_sec + conf->cacheexpiry;
+	    tlsaddcacrl(conf->tlsctx, conf);
+	}
+#endif
+#ifdef RADPROT_DTLS
+	if (conf->dtlsctx) {
+	    if (conf->dtlsexpiry)
+		conf->dtlsexpiry = now.tv_sec + conf->cacheexpiry;
+	    tlsaddcacrl(conf->dtlsctx, conf);
+	}
+#endif
+    }
+}
+
 X509 *verifytlscert(SSL *ssl) {
     X509 *cert;
     unsigned long error;
@@ -368,10 +540,10 @@ static int subjectaltnameaddr(X509 *cert, int family, struct in6_addr *addr) {
 	if (gn->type != GEN_IPADD)
 	    continue;
 	r = -1;
-	v = (char *)ASN1_STRING_data(gn->d.ia5);
+	v = (char *)ASN1_STRING_get0_data(gn->d.ia5);
 	l = ASN1_STRING_length(gn->d.ia5);
 	if (((family == AF_INET && l == sizeof(struct in_addr)) || (family == AF_INET6 && l == sizeof(struct in6_addr)))
-	    && !memcmp(v, &addr, l)) {
+	    && !memcmp(v, addr, l)) {
 	    r = 1;
 	    break;
 	}
@@ -382,7 +554,7 @@ static int subjectaltnameaddr(X509 *cert, int family, struct in6_addr *addr) {
 
 static int subjectaltnameregexp(X509 *cert, int type, char *exact,  regex_t *regex) {
     int loc, i, l, n, r = 0;
-    char *s, *v;
+    char *s, *v, *fail = NULL, *tmp;
     X509_EXTENSION *ex;
     STACK_OF(GENERAL_NAME) *alt;
     GENERAL_NAME *gn;
@@ -404,12 +576,12 @@ static int subjectaltnameregexp(X509 *cert, int type, char *exact,  regex_t *reg
 	if (gn->type != type)
 	    continue;
 	r = -1;
-	v = (char *)ASN1_STRING_data(gn->d.ia5);
+	v = (char *)ASN1_STRING_get0_data(gn->d.ia5);
 	l = ASN1_STRING_length(gn->d.ia5);
 	if (l <= 0)
 	    continue;
 #ifdef DEBUG
-	printfchars(NULL, gn->type == GEN_DNS ? "dns" : "uri", NULL, v, l);
+	printfchars(NULL, gn->type == GEN_DNS ? "dns" : "uri", NULL, (uint8_t *) v, l);
 #endif
 	if (exact) {
 	    if (memcmp(v, exact, l))
@@ -420,16 +592,26 @@ static int subjectaltnameregexp(X509 *cert, int type, char *exact,  regex_t *reg
 		debug(DBG_ERR, "malloc failed");
 		continue;
 	    }
-	    if (regexec(regex, s, 0, NULL, 0)) {
-		free(s);
-		continue;
+        debug(DBG_DBG, "subjectaltnameregex: matching %s", s);
+        if (regexec(regex, s, 0, NULL, 0)) {
+            tmp = fail;
+            if (asprintf(&fail, "%s%s%s", tmp ? tmp : "", tmp ? ", " : "", s) >= 0)
+                free(tmp);
+            else
+                fail = tmp;
+            free(s);
+            continue;
 	    }
 	    free(s);
 	}
 	r = 1;
 	break;
     }
+    if (r!=1)
+        debug(DBG_WARN, "subjectaltnameregex: no matching Subject Alt Name %s found! (%s)",
+            type == GEN_DNS ? "DNS" : "URI", fail);
     GENERAL_NAMES_free(alt);
+    free(fail);
     return r;
 }
 
@@ -448,7 +630,7 @@ static int cnregexp(X509 *cert, char *exact, regex_t *regex) {
 	    break;
 	e = X509_NAME_get_entry(nm, loc);
 	t = X509_NAME_ENTRY_get_data(e);
-	v = (char *) ASN1_STRING_data(t);
+	v = (char *) ASN1_STRING_get0_data(t);
 	l = ASN1_STRING_length(t);
 	if (l < 0)
 	    continue;
@@ -476,64 +658,73 @@ static int cnregexp(X509 *cert, char *exact, regex_t *regex) {
 int certnamecheck(X509 *cert, struct list *hostports) {
     struct list_node *entry;
     struct hostportres *hp;
-    int r;
+    int r = 0;
     uint8_t type = 0; /* 0 for DNS, AF_INET for IPv4, AF_INET6 for IPv6 */
     struct in6_addr addr;
 
     for (entry = list_first(hostports); entry; entry = list_next(entry)) {
-	hp = (struct hostportres *)entry->data;
-	if (hp->prefixlen != 255) {
-	    /* we disable the check for prefixes */
-	    return 1;
-	}
-	if (inet_pton(AF_INET, hp->host, &addr))
-	    type = AF_INET;
-	else if (inet_pton(AF_INET6, hp->host, &addr))
-	    type = AF_INET6;
-	else
-	    type = 0;
+        hp = (struct hostportres *)entry->data;
+        if (hp->prefixlen != 255) {
+            /* we disable the check for prefixes */
+            return 1;
+        }
+        if (inet_pton(AF_INET, hp->host, &addr))
+            type = AF_INET;
+        else if (inet_pton(AF_INET6, hp->host, &addr))
+            type = AF_INET6;
+        else
+            type = 0;
 
-	r = type ? subjectaltnameaddr(cert, type, &addr) : subjectaltnameregexp(cert, GEN_DNS, hp->host, NULL);
-	if (r) {
-	    if (r > 0) {
-		debug(DBG_DBG, "certnamecheck: Found subjectaltname matching %s %s", type ? "address" : "host", hp->host);
-		return 1;
-	    }
-	    debug(DBG_WARN, "certnamecheck: No subjectaltname matching %s %s", type ? "address" : "host", hp->host);
-	} else {
-	    if (cnregexp(cert, hp->host, NULL)) {
-		debug(DBG_DBG, "certnamecheck: Found cn matching host %s", hp->host);
-		return 1;
-	    }
-	    debug(DBG_WARN, "certnamecheck: cn not matching host %s", hp->host);
-	}
+        if (type)
+            r = subjectaltnameaddr(cert, type, &addr);
+        if (!r)
+            r = subjectaltnameregexp(cert, GEN_DNS, hp->host, NULL);
+        if (r) {
+            if (r > 0) {
+                debug(DBG_DBG, "certnamecheck: Found subjectaltname matching %s %s", type ? "address" : "host", hp->host);
+                return 1;
+            }
+            debug(DBG_WARN, "certnamecheck: No subjectaltname matching %s %s", type ? "address" : "host", hp->host);
+        } else {
+            if (cnregexp(cert, hp->host, NULL)) {
+                debug(DBG_DBG, "certnamecheck: Found cn matching host %s", hp->host);
+                return 1;
+            }
+            debug(DBG_WARN, "certnamecheck: cn not matching host %s", hp->host);
+        }
     }
     return 0;
 }
 
 int verifyconfcert(X509 *cert, struct clsrvconf *conf) {
+    char *subject;
+    int ok = 1;
+
+    subject = X509_NAME_oneline(X509_get_subject_name(cert), NULL, 0);
+    debug(DBG_DBG, "verifyconfcert: verify certificate for host %s, subject %s", conf->name, subject);
     if (conf->certnamecheck) {
-	if (!certnamecheck(cert, conf->hostports)) {
-	    debug(DBG_WARN, "verifyconfcert: certificate name check failed");
-	    return 0;
-	}
-	debug(DBG_WARN, "verifyconfcert: certificate name check ok");
+        debug(DBG_DBG, "verifyconfcert: verify hostname");
+        if (!certnamecheck(cert, conf->hostports)) {
+            debug(DBG_DBG, "verifyconfcert: certificate name check failed for host %s", conf->name);
+            ok = 0;
+        }
     }
     if (conf->certcnregex) {
-	if (cnregexp(cert, NULL, conf->certcnregex) < 1) {
-	    debug(DBG_WARN, "verifyconfcert: CN not matching regex");
-	    return 0;
-	}
-	debug(DBG_DBG, "verifyconfcert: CN matching regex");
+        debug(DBG_DBG, "verifyconfcert: matching CN regex %s", conf->matchcertattr);
+        if (cnregexp(cert, NULL, conf->certcnregex) < 1) {
+            debug(DBG_WARN, "verifyconfcert: CN not matching regex for host %s (%s)", conf->name, subject);
+            ok = 0;
+        }
     }
     if (conf->certuriregex) {
-	if (subjectaltnameregexp(cert, GEN_URI, NULL, conf->certuriregex) < 1) {
-	    debug(DBG_WARN, "verifyconfcert: subjectaltname URI not matching regex");
-	    return 0;
-	}
-	debug(DBG_DBG, "verifyconfcert: subjectaltname URI matching regex");
+        debug(DBG_DBG, "verifyconfcert: matching subjectaltname URI regex %s", conf->matchcertattr);
+        if (subjectaltnameregexp(cert, GEN_URI, NULL, conf->certuriregex) < 1) {
+            debug(DBG_WARN, "verifyconfcert: subjectaltname URI not matching regex for host %s (%s)", conf->name, subject);
+            ok = 0;
+        }
     }
-    return 1;
+    free(subject);
+    return ok;
 }
 
 int conftls_cb(struct gconffile **cf, void *arg, char *block, char *opt, char *val) {
@@ -584,6 +775,7 @@ int conftls_cb(struct gconffile **cf, void *arg, char *block, char *opt, char *v
 	debug(DBG_ERR, "conftls_cb: malloc failed");
 	goto errexit;
     }
+    pthread_mutex_init(&conf->lock, NULL);
 
     if (!tlsconfs)
 	tlsconfs = hash_create();
@@ -640,6 +832,127 @@ int addmatchcertattr(struct clsrvconf *conf) {
     }
     return 1;
 }
+
+int sslaccepttimeout (SSL *ssl, int timeout) {
+    int socket, origflags, ndesc, r = -1, sockerr = 0;
+    socklen_t errlen = sizeof(sockerr);
+    struct pollfd fds[1];
+    uint8_t want_write = 1;
+
+    socket = SSL_get_fd(ssl);
+    origflags = fcntl(socket, F_GETFL, 0);
+    if (origflags == -1) {
+        debugerrno(errno, DBG_WARN, "Failed to get flags");
+        return -1;
+    }
+    if (fcntl(socket, F_SETFL, origflags | O_NONBLOCK) == -1) {
+        debugerrno(errno, DBG_WARN, "Failed to set O_NONBLOCK");
+        return -1;
+    }
+
+    while (r < 1) {
+        fds[0].fd = socket;
+        fds[0].events = POLLIN;
+        if (want_write) {
+            fds[0].events |= POLLOUT;
+            want_write = 0;
+        }
+        if ((ndesc = poll(fds, 1, timeout * 1000)) < 1) {
+            if (ndesc == 0)
+                debug(DBG_DBG, "sslaccepttimeout: timeout during SSL_accept");
+            else
+                debugerrno(errno, DBG_DBG, "sslaccepttimeout: poll error");
+            break;
+        }
+
+        if (fds[0].revents & POLLERR) {
+            if(!getsockopt(socket, SOL_SOCKET, SO_ERROR, (void *)&sockerr, &errlen))
+                debug(DBG_WARN, "SSL Accept failed: %s", strerror(sockerr));
+            else
+                debug(DBG_WARN, "SSL Accept failed: unknown error");
+        } else if (fds[0].revents & POLLHUP) {
+                debug(DBG_WARN, "SSL Accept error: hang up");
+        } else if (fds[0].revents & POLLNVAL) {
+                debug(DBG_WARN, "SSL Accept error: fd not open");
+        } else {
+            r = SSL_accept(ssl);
+            if (r <= 0) {
+                switch (SSL_get_error(ssl, r)) {
+                    case SSL_ERROR_WANT_WRITE:
+                        want_write = 1;
+                    case SSL_ERROR_WANT_READ:
+                        continue;
+                }
+            }
+        }
+        break;
+    }
+
+    if (fcntl(socket, F_SETFL, origflags) == -1)
+        debugerrno(errno, DBG_WARN, "Failed to set original flags back");
+    return r;
+}
+
+int sslconnecttimeout(SSL *ssl, int timeout) {
+    int socket, origflags, ndesc, r = -1, sockerr = 0;
+    socklen_t errlen = sizeof(sockerr);
+    struct pollfd fds[1];
+    uint8_t want_write = 1;
+
+    socket = SSL_get_fd(ssl);
+    origflags = fcntl(socket, F_GETFL, 0);
+    if (origflags == -1) {
+        debugerrno(errno, DBG_WARN, "Failed to get flags");
+        return -1;
+    }
+    if (fcntl(socket, F_SETFL, origflags | O_NONBLOCK) == -1) {
+        debugerrno(errno, DBG_WARN, "Failed to set O_NONBLOCK");
+        return -1;
+    }
+
+    while (r < 1) {
+        fds[0].fd = socket;
+        fds[0].events = POLLIN;
+        if (want_write) {
+            fds[0].events |= POLLOUT;
+            want_write = 0;
+        }
+        if ((ndesc = poll(fds, 1, timeout * 1000)) < 1) {
+            if (ndesc == 0)
+                debug(DBG_DBG, "sslconnecttimeout: timeout during SSL_connect");
+            else
+                debugerrno(errno, DBG_DBG, "sslconnecttimeout: poll error");
+            break;
+        }
+
+        if (fds[0].revents & POLLERR) {
+            if(!getsockopt(socket, SOL_SOCKET, SO_ERROR, (void *)&sockerr, &errlen))
+                debug(DBG_WARN, "SSL Connection failed: %s", strerror(sockerr));
+            else
+                debug(DBG_WARN, "SSL Connection failed: unknown error");
+        } else if (fds[0].revents & POLLHUP) {
+                debug(DBG_WARN, "SSL Connect error: hang up");
+        } else if (fds[0].revents & POLLNVAL) {
+                debug(DBG_WARN, "SSL Connect error: fd not open");
+        } else {
+            r = SSL_connect(ssl);
+            if (r <= 0) {
+                switch (SSL_get_error(ssl, r)) {
+                    case SSL_ERROR_WANT_WRITE:
+                        want_write = 1;
+                    case SSL_ERROR_WANT_READ:
+                        continue;
+                }
+            }
+        }
+        break;
+    }
+
+    if (fcntl(socket, F_SETFL, origflags) == -1)
+        debugerrno(errno, DBG_WARN, "Failed to set original flags back");
+    return r;
+}
+
 #else
 /* Just to makes file non-empty, should rather avoid compiling this file when not needed */
 static void tlsdummy() {
