@@ -1,18 +1,10 @@
-/*
- * Copyright (C) 2006-2008 Stig Venaas <venaas@uninett.no>
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- */
-
-/* Code contributions from:
- *
- * Stefan Winter <stefan.winter@restena.lu>
- */
+/* Copyright (c) 2007-2009, UNINETT AS
+ * Copyright (c) 2016, NORDUnet A/S */
+/* See LICENSE for licensing information. */
 
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include <netinet/tcp.h>
 #include <netdb.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -20,8 +12,9 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <sys/select.h>
+#include <poll.h>
 #include <stdarg.h>
+#include <assert.h>
 #include "debug.h"
 #include "util.h"
 
@@ -33,13 +26,13 @@ char *stringcopy(const char *s, int len) {
 	len = strlen(s);
     r = malloc(len + 1);
     if (!r)
-	debug(DBG_ERR, "stringcopy: malloc failed");
+	debugx(1, DBG_ERR, "stringcopy: malloc failed");
     memcpy(r, s, len);
     r[len] = '\0';
     return r;
 }
 
-void printfchars(char *prefixfmt, char *prefix, char *charfmt, char *chars, int len) {
+void printfchars(char *prefixfmt, char *prefix, char *charfmt, uint8_t *chars, int len) {
     int i;
     unsigned char *s = (unsigned char *)chars;
     if (prefix)
@@ -66,32 +59,28 @@ struct sockaddr *addr_copy(struct sockaddr *in) {
     switch (in->sa_family) {
     case AF_INET:
 	out = malloc(sizeof(struct sockaddr_in));
-	if (out) {
-	    memset(out, 0, sizeof(struct sockaddr_in));
-	    ((struct sockaddr_in *)out)->sin_addr = ((struct sockaddr_in *)in)->sin_addr;
-	}
+        if (out == NULL)
+            return NULL;
+        *(struct sockaddr_in *)out = *(struct sockaddr_in *)in;
 	break;
     case AF_INET6:
 	out = malloc(sizeof(struct sockaddr_in6));
-	if (out) {
-	    memset(out, 0, sizeof(struct sockaddr_in6));
-	    ((struct sockaddr_in6 *)out)->sin6_addr = ((struct sockaddr_in6 *)in)->sin6_addr;
-	}
+        if (out == NULL)
+            return NULL;
+        *(struct sockaddr_in6 *)out = *(struct sockaddr_in6 *)in;
 	break;
     }
-    out->sa_family = in->sa_family;
+    assert(out);
 #ifdef SIN6_LEN
     out->sa_len = in->sa_len;
 #endif
     return out;
 }
 
-char *addr2string(struct sockaddr *addr) {
+const char *addr2string(struct sockaddr *addr, char *buf, size_t len) {
     struct sockaddr_in6 *sa6;
     struct sockaddr_in sa4;
-    static char addr_buf[2][INET6_ADDRSTRLEN];
-    static int i = 0;
-    i = !i;
+
     if (addr->sa_family == AF_INET6) {
 	sa6 = (struct sockaddr_in6 *)addr;
 	if (IN6_IS_ADDR_V4MAPPED(&sa6->sin6_addr)) {
@@ -102,46 +91,14 @@ char *addr2string(struct sockaddr *addr) {
 	    addr = (struct sockaddr *)&sa4;
 	}
     }
-    if (getnameinfo(addr, SOCKADDRP_SIZE(addr), addr_buf[i], sizeof(addr_buf[i]),
+    if (getnameinfo(addr, SOCKADDRP_SIZE(addr), buf, len,
                     NULL, 0, NI_NUMERICHOST)) {
         debug(DBG_WARN, "getnameinfo failed");
         return "getnameinfo_failed";
     }
-    return addr_buf[i];
+    return buf;
 }
 
-#if 0
-/* not in use */
-int connectport(int type, char *host, char *port) {
-    struct addrinfo hints, *res0, *res;
-    int s = -1;
-
-    memset(&hints, 0, sizeof(hints));
-    hints.ai_socktype = type;
-    hints.ai_family = AF_UNSPEC;
-
-    if (getaddrinfo(host, port, &hints, &res0) != 0) {
-	debug(DBG_ERR, "connectport: can't resolve host %s port %s", host, port);
-	return -1;
-    }
-
-    for (res = res0; res; res = res->ai_next) {
-	s = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
-	if (s < 0) {
-	    debug(DBG_WARN, "connectport: socket failed");
-	    continue;
-	}
-	if (connect(s, res->ai_addr, res->ai_addrlen) == 0)
-	    break;
-	debug(DBG_WARN, "connectport: connect failed");
-	close(s);
-	s = -1;
-    }
-    freeaddrinfo(res0);
-    return s;
-}
-#endif
-
 /* Disable the "Don't Fragment" bit for UDP sockets. It is set by default, which may cause an "oversized"
    RADIUS packet to be discarded on first attempt (due to Path MTU discovery).
 */
@@ -164,7 +121,33 @@ void disable_DF_bit(int socket, struct addrinfo *res) {
     }
 }
 
-int bindtoaddr(struct addrinfo *addrinfo, int family, int reuse, int v6only) {
+void enable_keepalive(int socket) {
+    int optval;
+    socklen_t optlen = sizeof(optval);
+
+#if !defined(TCP_KEEPCNT) || !defined(TCP_KEEPIDLE) || !defined(TCP_KEEPINTVL)
+    debug(DBG_NOTICE, "TCP Keepalive feature might be limited on this platform");
+#else
+    optval = 3;
+    if(setsockopt(socket, SOL_TCP, TCP_KEEPCNT, &optval, optlen) < 0) {
+        debug(DBG_ERR, "enable_keepalive: setsockopt TCP_KEEPCNT failed");
+    }
+    optval = 10;
+    if(setsockopt(socket, SOL_TCP, TCP_KEEPIDLE, &optval, optlen) < 0) {
+        debug(DBG_ERR, "enable_keepalive: setsockopt TCP_KEEPIDLE %d failed", optval);
+    }
+    optval = 10;
+    if(setsockopt(socket, SOL_TCP, TCP_KEEPINTVL, &optval, optlen) < 0) {
+        debug(DBG_ERR, "enable_keepalive: setsockopt TCP_KEEPINTVL failed");
+    }
+#endif
+    optval = 1;
+    if(setsockopt(socket, SOL_SOCKET, SO_KEEPALIVE, &optval, optlen) < 0) {
+        debug(DBG_ERR, "enable_keepalive: setsockopt SO_KEEPALIVE failed");
+    }
+}
+
+int bindtoaddr(struct addrinfo *addrinfo, int family, int reuse) {
     int s, on = 1;
     struct addrinfo *res;
 
@@ -180,10 +163,12 @@ int bindtoaddr(struct addrinfo *addrinfo, int family, int reuse, int v6only) {
 	disable_DF_bit(s,res);
 
 	if (reuse)
-	    setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
+	    if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) == -1)
+                debugerrno(errno, DBG_WARN, "Failed to set SO_REUSEADDR");
 #ifdef IPV6_V6ONLY
-	if (v6only)
-	    setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on));
+	if (family == AF_INET6)
+	    if (setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on)) == -1)
+                debugerrno(errno, DBG_WARN, "Failed to set IPV6_V6ONLY");
 #endif
 	if (!bind(s, res->ai_addr, res->ai_addrlen))
 	    return s;
@@ -193,13 +178,20 @@ int bindtoaddr(struct addrinfo *addrinfo, int family, int reuse, int v6only) {
     return -1;
 }
 
-int connectnonblocking(int s, const struct sockaddr *addr, socklen_t addrlen, struct timeval *timeout) {
-    int origflags, error = 0, r = -1;
-    fd_set writefds;
-    socklen_t len;
+int connectnonblocking(int s, const struct sockaddr *addr, socklen_t addrlen, int timeout) {
+    int origflags, r = -1, sockerr = 0;
+    socklen_t errlen = sizeof(sockerr);
+    struct pollfd fds[1];
 
     origflags = fcntl(s, F_GETFL, 0);
-    fcntl(s, F_SETFL, origflags | O_NONBLOCK);
+    if (origflags == -1) {
+        debugerrno(errno, DBG_WARN, "Failed to get flags");
+        return -1;
+    }
+    if (fcntl(s, F_SETFL, origflags | O_NONBLOCK) == -1) {
+        debugerrno(errno, DBG_WARN, "Failed to set O_NONBLOCK");
+        return -1;
+    }
     if (!connect(s, addr, addrlen)) {
 	r = 0;
 	goto exit;
@@ -207,41 +199,49 @@ int connectnonblocking(int s, const struct sockaddr *addr, socklen_t addrlen, st
     if (errno != EINPROGRESS)
 	goto exit;
 
-    FD_ZERO(&writefds);
-    FD_SET(s, &writefds);
-    if (select(s + 1, NULL, &writefds, NULL, timeout) < 1)
+    fds[0].fd = s;
+    fds[0].events = POLLOUT;
+    if (poll(fds, 1, timeout * 1000) < 1)
 	goto exit;
 
-    len = sizeof(error);
-    if (!getsockopt(s, SOL_SOCKET, SO_ERROR, (char*)&error, &len) && !error)
-	r = 0;
+    if (fds[0].revents & POLLERR) {
+        if(!getsockopt(s, SOL_SOCKET, SO_ERROR, (void *)&sockerr, &errlen))
+            debug(DBG_WARN, "Connection failed: %s", strerror(sockerr));
+        else
+            debug(DBG_WARN, "Connection failed: unknown error");
+    } else if (fds[0].revents & POLLHUP) {
+            debug(DBG_WARN, "Connect error: hang up");
+    } else if (fds[0].revents & POLLNVAL) {
+            debug(DBG_WARN, "Connect error: fd not open");
+    } else if(fds[0].revents & POLLOUT) {
+        debug(DBG_DBG, "Connection up");
+        r = 0;
+    }
 
 exit:
-    fcntl(s, F_SETFL, origflags);
+    if (fcntl(s, F_SETFL, origflags) == -1)
+        debugerrno(errno, DBG_WARN, "Failed to set original flags back");
     return r;
 }
 
 int connecttcp(struct addrinfo *addrinfo, struct addrinfo *src, uint16_t timeout) {
     int s;
     struct addrinfo *res;
-    struct timeval to;
 
     s = -1;
     if (timeout) {
 	if (addrinfo && addrinfo->ai_next && timeout > 5)
 	    timeout = 5;
-	to.tv_sec = timeout;
-	to.tv_usec = 0;
     }
 
     for (res = addrinfo; res; res = res->ai_next) {
-	s = bindtoaddr(src, res->ai_family, 1, 1);
+	s = bindtoaddr(src, res->ai_family, 1);
 	if (s < 0) {
 	    debug(DBG_WARN, "connecttoserver: socket failed");
 	    continue;
 	}
 	if ((timeout
-	     ? connectnonblocking(s, res->ai_addr, res->ai_addrlen, &to)
+	     ? connectnonblocking(s, res->ai_addr, res->ai_addrlen, timeout)
 	     : connect(s, res->ai_addr, res->ai_addrlen)) == 0)
 	    break;
 	debug(DBG_WARN, "connecttoserver: connect failed");
