@@ -1,3 +1,7 @@
+/* Copyright (c) 2007-2009, UNINETT AS
+ * Copyright (c) 2016-2017, NORDUnet A/S */
+/* See LICENSE for licensing information. */
+
 #include <sys/socket.h>
 #include <netdb.h>
 
@@ -10,13 +14,14 @@
 			      sizeof(struct sockaddr_in6))
 
 char *stringcopy(const char *s, int len);
-char *addr2string(struct sockaddr *addr);
+const char *addr2string(struct sockaddr *addr, char *buf, size_t len);
 struct sockaddr *addr_copy(struct sockaddr *in);
 void port_set(struct sockaddr *sa, uint16_t port);
 
-void printfchars(char *prefixfmt, char *prefix, char *charfmt, char *chars, int len);
+void printfchars(char *prefixfmt, char *prefix, char *charfmt, uint8_t *chars, int len);
 void disable_DF_bit(int socket, struct addrinfo *res);
-int bindtoaddr(struct addrinfo *addrinfo, int family, int reuse, int v6only);
+void enable_keepalive(int socket);
+int bindtoaddr(struct addrinfo *addrinfo, int family, int reuse);
 int connecttcp(struct addrinfo *addrinfo, struct addrinfo *src, uint16_t timeout);
 
 
